@@ -1,11 +1,11 @@
 import cPickle, os, traceback
 from urlparse import urlparse
 from error import log, logtb, logparam
-
+import pygtk
+pygtk.require('2.0')
 import gconf
-
-import Event as event
-import LookupManager as lookupmanager
+import Event
+import LookupManager
 
 GCONF_STRAW_ROOT = "/apps/straw"
 
@@ -102,6 +102,9 @@ class ConfigPersistence:
 ################
 # Proxy config
 class BaseProxyConfig(object):
+    def __init__(self):
+        self._config_working = False
+        
     def get_config_working(self):
         return self._config_working
 
@@ -113,9 +116,11 @@ class BaseProxyConfig(object):
     def set_host(self, host):
         self._host = host
         self._ip = None
-        if self._host:
-            lookupmanager.get_instance().lookup(
-                self._host, self.ip_lookup_callback)
+
+    def lookup_host(self):
+        if self._host and not self._ip:
+            LookupManager.get_instance().lookup(
+                self._host, self._ip_lookup_callback)
 
     host = property(get_host, set_host, None, "")
 
@@ -130,7 +135,7 @@ class BaseProxyConfig(object):
 
     ip = property(get_ip, set_ip, None, "")
 
-    def ip_lookup_callback(self, host, ip, data):
+    def _ip_lookup_callback(self, host, ip, data):
         self.ip = ip
 
     def _is_waiting(self):
@@ -152,6 +157,7 @@ class GconfProxyConfig(BaseProxyConfig):
     GCONF_HTTP_PROXY_PASSWORD = GCONF_HTTP_PROXY_ROOT + "/authentication_password"
 
     def __init__(self):
+        BaseProxyConfig.__init__(self)
         client = gconf.client_get_default()
         self._config_working = client.dir_exists(self.GCONF_PROXY_ROOT) and client.dir_exists(self.GCONF_HTTP_PROXY_ROOT)
         if not self._config_working:
@@ -211,6 +217,7 @@ class EnvironmentProxyConfig(BaseProxyConfig):
     """Encapsulate proxy use and location information, environment reading"""
 
     def __init__(self):
+        BaseProxyConfig.__init__(self)
         try:
             result = self._read_env()
         except:
@@ -251,6 +258,7 @@ class EnvironmentProxyConfig(BaseProxyConfig):
 
 class NullProxyConfig(BaseProxyConfig):
     def __init__(self):
+        BaseProxyConfig.__init__(self)
         self.use = 0
         self._config_working = 0
         self._host = None
@@ -262,16 +270,16 @@ class NullProxyConfig(BaseProxyConfig):
 
 ###################
 # The main man
-class Config(object, event.SignalEmitter):
+class Config(object, Event.SignalEmitter):
     _straw_dir = os.path.join(os.getenv("HOME"), ".straw")
     _straw_config_file = os.path.join(_straw_dir, "config")
 
     def __init__(self, persistence):
-        event.SignalEmitter.__init__(self)
+        Event.SignalEmitter.__init__(self)
         self.persistence = persistence
-        self.initialize_slots(event.ItemOrderChangedSignal,
-                              event.OfflineModeChangedSignal,
-                              event.PollFrequencyChangedSignal)
+        self.initialize_slots(Event.ItemOrderChangedSignal,
+                              Event.OfflineModeChangedSignal,
+                              Event.PollFrequencyChangedSignal)
         pcchoices = (GconfProxyConfig, EnvironmentProxyConfig, NullProxyConfig)
         for p in pcchoices:
             self._proxy_config = p()
@@ -298,6 +306,8 @@ class Config(object, event.SignalEmitter):
             OPTION_SUB_PANE_POS)
         self._offline = self.persistence.load_option(OPTION_OFFLINE)
         self._feed_info_visibility = self.persistence.load_option(OPTION_FEED_INFO_VISIBILITY)
+        self._use_threads = True
+        self._reload_css = False
 
     def get_straw_dir(self):
         return self._straw_dir
@@ -339,7 +349,7 @@ class Config(object, event.SignalEmitter):
         if self._poll_freq != poll_frequency:
             self._poll_freq = poll_frequency
             self.persistence.save_option(OPTION_POLL_FREQUENCY, poll_frequency)
-            self.emit_signal(event.PollFrequencyChangedSignal(self, poll_frequency))
+            self.emit_signal(Event.PollFrequencyChangedSignal(self, poll_frequency))
         return
 
     def get_poll_frequency(self):
@@ -365,7 +375,7 @@ class Config(object, event.SignalEmitter):
         if self._items_stored != num:
             self._items_stored = num
             self.persistence.save_option(OPTION_ITEMS_STORED, num)
-            self.emit_signal(event.NumberOfItemsStoredChangedSignal(self))
+            self.emit_signal(Event.NumberOfItemsStoredChangedSignal(self))
         return
 
     number_of_items_stored = property(get_number_of_items_stored,
@@ -376,7 +386,7 @@ class Config(object, event.SignalEmitter):
         if self._item_order != order:
             self._item_order = order
             self.persistence.save_option(OPTION_ITEM_ORDER, order)
-            self.emit_signal(event.ItemOrderChangedSignal(self))
+            self.emit_signal(Event.ItemOrderChangedSignal(self))
         return
 
     def get_item_order(self):
@@ -416,7 +426,7 @@ class Config(object, event.SignalEmitter):
             self._main_window_size = size
             self.persistence.save_option(OPTION_WINDOW_SIZE_W, size[0])
             self.persistence.save_option(OPTION_WINDOW_SIZE_H, size[1])
-            self.emit_signal(event.MainWindowSizeChangedSignal(self))
+            self.emit_signal(Event.MainWindowSizeChangedSignal(self))
         return
 
     def get_main_window_size(self):
@@ -428,7 +438,7 @@ class Config(object, event.SignalEmitter):
         if self._offline != mode:
             self._offline = mode
             self.persistence.save_option(OPTION_OFFLINE, mode)
-            self.emit_signal(event.OfflineModeChangedSignal(self))
+            self.emit_signal(Event.OfflineModeChangedSignal(self))
         return
 
     def get_offline(self):
@@ -440,7 +450,7 @@ class Config(object, event.SignalEmitter):
         if self._main_pane_position != position:
             self._main_pane_position = position
             self.persistence.save_option(OPTION_MAIN_PANE_POS, position)
-            self.emit_signal(event.MainPanePositionChangedSignal(self))
+            self.emit_signal(Event.MainPanePositionChangedSignal(self))
 
     def get_main_pane_position(self):
         return self._main_pane_position
@@ -451,44 +461,78 @@ class Config(object, event.SignalEmitter):
         if self._sub_pane_position != position:
             self._sub_pane_position = position
             self.persistence.save_option(OPTION_SUB_PANE_POS, position)
-            self.emit_signal(event.SubPanePositionChangedSignal(self))
+            self.emit_signal(Event.SubPanePositionChangedSignal(self))
 
     def get_sub_pane_position(self):
         return self._sub_pane_position
 
     sub_pane_position = property(get_sub_pane_position, set_sub_pane_position, None, "")
 
-config_instance = None
+    def set_use_threads(self, threads):
+        self._use_threads = threads
+
+    def get_use_threads(self):
+        return self._use_threads
+
+    use_threads = property(get_use_threads, set_use_threads)
+
+    def set_reload_css(self, reload): self._reload_css = reload
+    def get_reload_css(self): return self._reload_css
+    reload_css = property(get_reload_css, set_reload_css)
 
 def convert_if_necessary(config):
     if not os.path.exists(config.straw_config_file):
         return
 
-    f = open(config.straw_config_file, "r")
-    cf = cPickle.load(f)
-    if cf.has_key('poll_frequency'):
-        config.poll_frequency = cf.get('poll_frequency')
-        config.number_of_items_stored = cf.get('number_of_items_stored')
-        config.item_order = cf.get('item_order')
-        config.main_window_size = cf.get('main_window_size')
-        config.main_pane_position = cf.get('main_pane_position')
-        config.sub_pane_position = cf.get('sub_pane_position')
-        config.offline = cf.get('offline')
+    try:
+        f = open(config.straw_config_file, "r")
+        cf = cPickle.load(f)
 
-        del cf['poll_frequency']
-        del cf['number_of_items_stored']
-        del cf['item_order']
-        del cf['main_window_size']
-        del cf['main_pane_position']
-        del cf['sub_pane_position']
-        del cf['offline']
+        feeds = cf.get('feeds', None)
+        if feeds is not None and feeds[0].get('_n_items_unread', None) is None:
+            import ItemStore
 
+            def _convert_feeds(feeds, parent):
+                itemstore = ItemStore.get_instance(config.straw_dir)
+
+                for feed in feeds:
+                    if isinstance(feed, list):
+                        _convert_feeds(feed[1:], parent)
+                    else:
+                        stored = feed['_items_stored']
+                        if stored > -1:
+                            cutoff = stored
+                        else:
+                            cutoff = config.number_of_items_stored
+                        feed['_n_items_unread'] = itemstore.get_number_of_unread(feed['_id'], cutoff)
+                return feeds
+
+            config.feeds = _convert_feeds(feeds, None)
+
+        if cf.has_key('poll_frequency'):
+            config.poll_frequency = cf.get('poll_frequency')
+            config.number_of_items_stored = cf.get('number_of_items_stored')
+            config.item_order = cf.get('item_order')
+            config.main_window_size = cf.get('main_window_size')
+            config.main_pane_position = cf.get('main_pane_position')
+            config.sub_pane_position = cf.get('sub_pane_position')
+            config.offline = cf.get('offline')
+
+            del cf['poll_frequency']
+            del cf['number_of_items_stored']
+            del cf['item_order']
+            del cf['main_window_size']
+            del cf['main_pane_position']
+            del cf['sub_pane_position']
+            del cf['offline']
+
+            f.close()
+            f = open(config.straw_config_file, "w")
+            f.seek(0)
+            f.truncate()
+            cPickle.dump(cf, f, True)
+    finally:
         f.close()
-        f = open(config.straw_config_file, "w")
-        f.seek(0)
-        f.truncate()
-        cPickle.dump(cf, f, True)
-    f.close()
     return
 
 def create_gconf_persistence():
@@ -513,6 +557,7 @@ def create_instance():
     convert_if_necessary(config)
     return config
 
+config_instance = None
 def get_instance():
     global config_instance
     if config_instance is None:
