@@ -1,18 +1,24 @@
-import straw
+from Feed import Feed
+from StringIO import StringIO
+import Config
+import Event
+import FeedList
+import OPMLImport
 import error
 import locale
-from StringIO import StringIO
+import utils
 
 PSEUDO_ALL_KEY = 'ALL'
 PSEUDO_UNCATEGORIZED_KEY = 'UNCATEGORIZED'
 
-class FeedCategoryList(object, straw.SignalEmitter):
+class FeedCategoryList(object, Event.SignalEmitter):
     def __init__(self):
-        straw.SignalEmitter.__init__(self)
-        self.initialize_slots(straw.FeedCategoryChangedSignal,
-                              straw.FeedCategoryListChangedSignal,
-                              straw.FeedCategoryAddedSignal,
-                              straw.FeedCategoryRemovedSignal)
+        Event.SignalEmitter.__init__(self)
+        self.initialize_slots(Event.FeedCategoryChangedSignal,
+                              Event.FeedCategoryListChangedSignal,
+                              Event.FeedCategoryAddedSignal,
+                              Event.FeedCategoryRemovedSignal,
+                              Event.FeedCategoryListLoadedSignal)
         # have to define this here so the titles can be translated
         PSEUDO_TITLES = {PSEUDO_ALL_KEY: _('All'),
                          PSEUDO_UNCATEGORIZED_KEY: _('Uncategorized')}
@@ -23,10 +29,10 @@ class FeedCategoryList(object, straw.SignalEmitter):
         self._user_categories = []
         self._pseudo_categories = (self._un_category, self._all_category)
         self._loading = False
-        self._feedlist = straw.FeedList.get_instance()
-        self._feedlist.signal_connect(straw.FeedDeletedSignal,
+        self._feedlist = FeedList.get_instance()
+        self._feedlist.signal_connect(Event.FeedDeletedSignal,
                                 self.feed_deleted)
-        self._feedlist.signal_connect(straw.FeedCreatedSignal,
+        self._feedlist.signal_connect(Event.FeedCreatedSignal,
                                 self.feed_created)
 
     def load_data(self):
@@ -34,7 +40,7 @@ class FeedCategoryList(object, straw.SignalEmitter):
         Data format: [[{'title': '', 'subscription': {}, 'pseudo': pseudo key},
         {'id', feed_id1, 'from_subscription': bool} ...], ...]
         """
-        cats = straw.Config.get_instance().categories or []
+        cats = Config.get_instance().categories or []
         categorized = {}
         for c in cats:
             head = c[0]
@@ -68,7 +74,7 @@ class FeedCategoryList(object, straw.SignalEmitter):
                     categorized[feed] = True
             # User categories: connect pseudos later
             if not pseudo_key:
-                fc.signal_connect(straw.FeedCategoryChangedSignal,
+                fc.signal_connect(Event.FeedCategoryChangedSignal,
                                   self.category_changed)
                 self._user_categories.append(fc)
         # just in case we've missed any feeds, go through the list
@@ -90,10 +96,11 @@ class FeedCategoryList(object, straw.SignalEmitter):
            self.save_data()
         for cat in self.pseudo_categories:
             cat.signal_connect(
-                straw.FeedCategoryChangedSignal, self.pseudo_category_changed)
+                Event.FeedCategoryChangedSignal, self.pseudo_category_changed)
+        self.emit_signal(Event.FeedCategoryListLoadedSignal(self))
 
     def save_data(self):
-        straw.Config.get_instance().categories = [
+        Config.get_instance().categories = [
             cat.dump() for cat in self]
 
     def pseudo_category_changed(self, signal):
@@ -123,7 +130,6 @@ class FeedCategoryList(object, straw.SignalEmitter):
                 c.remove_feed(signal.feed)
             except ValueError:
                 pass
-        #self.emit_signal(straw.FeedCategoriesChangedSignal(self))
 
     def feed_created(self, signal):
         value = signal.feed
@@ -193,31 +199,31 @@ class FeedCategoryList(object, straw.SignalEmitter):
         return self.CategoryIterator(self)
 
     def add_category(self, category):
-        category.signal_connect(straw.FeedCategoryChangedSignal,
+        category.signal_connect(Event.FeedCategoryChangedSignal,
                                 self.category_changed)
         self._user_categories.append(category)
         self._user_categories.sort(lambda a, b: locale.strcoll(a.title.lower(), b.title.lower()))
-        self.emit_signal(straw.FeedCategoryAddedSignal(self, category))
+        self.emit_signal(Event.FeedCategoryAddedSignal(self, category))
         self.save_data()
 
     def remove_category(self, category):
-        category.signal_disconnect(straw.FeedCategoryChangedSignal,
+        category.signal_disconnect(Event.FeedCategoryChangedSignal,
                                    self.category_changed)
         self._user_categories.remove(category)
-        self.emit_signal(straw.FeedCategoryRemovedSignal(self, category))
+        self.emit_signal(Event.FeedCategoryRemovedSignal(self, category))
         self.save_data()
 
 # It might be good to have a superclass FeedCategorySubscription or something
 # so we could support different formats. However, I don't know of any other
 # relevant format used for this purpose, so that can be done later if needed.
 # Of course, they could also just implement the same interface.
-class OPMLCategorySubscription(object, straw.SignalEmitter):
+class OPMLCategorySubscription(object, Event.SignalEmitter):
     REFRESH_DEFAULT = -1
     
     def __init__(self, location=None):
-        straw.SignalEmitter.__init__(self)
-        self.initialize_slots(straw.FeedCategoryChangedSignal,
-                              straw.SubscriptionContentsUpdatedSignal)
+        Event.SignalEmitter.__init__(self)
+        self.initialize_slots(Event.FeedCategoryChangedSignal,
+                              Event.SubscriptionContentsUpdatedSignal)
         self._location = location
         self._username = None
         self._password = None
@@ -231,7 +237,7 @@ class OPMLCategorySubscription(object, straw.SignalEmitter):
 
     def set_location(self, location):
         self._location = location
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     location = property(get_location, set_location)
 
@@ -239,7 +245,7 @@ class OPMLCategorySubscription(object, straw.SignalEmitter):
 
     def set_username(self, username):
         self._username = username
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     username = property(get_username, set_username)
 
@@ -247,7 +253,7 @@ class OPMLCategorySubscription(object, straw.SignalEmitter):
 
     def set_password(self, password):
         self._password = password
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     password = property(get_password, set_password)
 
@@ -255,7 +261,7 @@ class OPMLCategorySubscription(object, straw.SignalEmitter):
 
     def set_previous_etag(self, etag):
         self._previous_etag = etag
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     previous_etag = property(get_previous_etag, set_previous_etag)
 
@@ -263,30 +269,30 @@ class OPMLCategorySubscription(object, straw.SignalEmitter):
 
     def set_frequency(self, frequency):
         self._frequency = frequency
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     frequency = property(get_frequency, set_frequency)
 
     def get_last_poll(self): return self._last_poll
     def set_last_poll(self, last_poll):
         self._last_poll = last_poll
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
     last_poll = property(get_last_poll, set_last_poll)
 
     def get_error(self): return self._error
     def set_error(self, error):
         self._error = error
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
     error = property(get_error, set_error)
 
     def parse(self, data):
         datastream = StringIO(data)
-        entries = straw.OPMLImport.read(datastream)
+        entries = OPMLImport.read(datastream)
         contents = [(e.url, e.text) for e in entries]
         updated = contents == self._contents
         self._contents = contents
         if updated:
-            self.emit_signal(straw.SubscriptionContentsUpdatedSignal(self))
+            self.emit_signal(Event.SubscriptionContentsUpdatedSignal(self))
 
     def get_contents(self):
         return self._contents
@@ -335,11 +341,11 @@ class CategoryMember(object):
     def set_from_subscription(self, p): self._from_subscription = p
     from_subscription = property(get_from_subscription, set_from_subscription)
 
-class FeedCategory(list, straw.SignalEmitter):
+class FeedCategory(list, Event.SignalEmitter):
     def __init__(self, title=""):
-        straw.SignalEmitter.__init__(self)
-        self.initialize_slots(straw.FeedCategoryChangedSignal,
-                              straw.SaveFeedsSignal)
+        Event.SignalEmitter.__init__(self)
+        self.initialize_slots(Event.FeedCategoryChangedSignal,
+                              Event.SaveFeedsSignal)
         self._title = title
         self._subscription = None
 
@@ -348,7 +354,7 @@ class FeedCategory(list, straw.SignalEmitter):
 
     def set_title(self, title):
         self._title = title
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     title = property(get_title, set_title)
 
@@ -357,15 +363,14 @@ class FeedCategory(list, straw.SignalEmitter):
             return
         subfeeds = self.subscription.contents
         sfdict = dict(subfeeds)
-        feedlist = straw.FeedList.get_instance()
+        feedlist = FeedList.get_instance()
         current = dict([(feed.location, feed) for feed in self.feeds])
         allfeeds = dict([(feed.location, feed) for feed in feedlist])
-        common, toadd, toremove = straw.utils.listdiff(sfdict.keys(),
-                                                       current.keys())
-        existing, nonexisting, ignore = straw.utils.listdiff(
+        common, toadd, toremove = utils.listdiff(sfdict.keys(), current.keys())
+        existing, nonexisting, ignore = utils.listdiff(
             toadd, allfeeds.keys())
         for f in nonexisting:
-            feed = straw.create_new_feed(sfdict[f], f)
+            feed = Feed.create_new_feed(sfdict[f], f)
             feedlist.append(None, feed)
             self.append_feed(feed, True)
         for f in existing:
@@ -378,7 +383,7 @@ class FeedCategory(list, straw.SignalEmitter):
         return
 
     def _subscription_changed(self, signal):
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     def _subscription_contents_updated(self, signal):
         self.read_contents_from_subscription()
@@ -389,17 +394,17 @@ class FeedCategory(list, straw.SignalEmitter):
     def set_subscription(self, sub):
         if self._subscription is not None:
             self._subscription.signal_disconnect(
-                straw.FeedCategoryChangedSignal, self._subscription_changed)
+                Event.FeedCategoryChangedSignal, self._subscription_changed)
             self._subscription.signal_disconnect(
-                straw.SubscriptionContentsUpdatedSignal,
+                Event.SubscriptionContentsUpdatedSignal,
                 self._subscription_contents_updated)
         if sub is not None:
-            sub.signal_connect(straw.FeedCategoryChangedSignal,
+            sub.signal_connect(Event.FeedCategoryChangedSignal,
                                self._subscription_changed)
-            sub.signal_connect(straw.SubscriptionContentsUpdatedSignal,
+            sub.signal_connect(Event.SubscriptionContentsUpdatedSignal,
                                self._subscription_contents_updated)
         self._subscription = sub
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     subscription = property(get_subscription, set_subscription)
 
@@ -412,24 +417,24 @@ class FeedCategory(list, straw.SignalEmitter):
     def append(self, value):
         error.log("warning, probably should be using append_feed?")
         list.append(self, value)
-        self.emit_signal(straw.FeedCategoryChangedSignal(self, feed=value.feed))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self, feed=value.feed))
 
     def append_feed(self, value, from_sub):
         list.append(self, CategoryMember(value, from_sub))
-        self.emit_signal(straw.FeedCategoryChangedSignal(self, feed=value))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self, feed=value))
 
     def insert(self, index, value):
         error.log("warning, probably should be using insert_feed?")
         list.insert(self, index, value)
-        self.emit_signal(straw.FeedCategoryChangedSignal(self, feed=value.feed))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self, feed=value.feed))
 
     def insert_feed(self, index, value, from_sub):
         list.insert(self, index, CategoryMember(value, from_sub))
-        self.emit_signal(straw.FeedCategoryChangedSignal(self, feed=value))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self, feed=value))
 
     def remove(self, value):
         list.remove(self, value)
-        self.emit_signal(straw.FeedCategoryChangedSignal(self, feed=value.feed))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self, feed=value.feed))
 
     def remove_feed(self, value):
         for index in range(len(self)):
@@ -438,11 +443,11 @@ class FeedCategory(list, straw.SignalEmitter):
                 break
         else:
             raise ValueError(value)
-        self.emit_signal(straw.FeedCategoryChangedSignal(self, feed=value))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self, feed=value))
 
     def reverse(self):
         list.reverse(self)
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     def index_feed(self, value):
         for index in range(len(self)):
@@ -462,7 +467,7 @@ class FeedCategory(list, straw.SignalEmitter):
                 a.feed.title.lower(), b.feed.title.lower()))
             for i in range(len(items)):
                 list.__setitem__(self, indices[i], items[i])
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     def move_feed(self, source, target):
         if target > source:
@@ -472,7 +477,7 @@ class FeedCategory(list, straw.SignalEmitter):
         t = self[source]
         del self[source]
         list.insert(self, target, t)
-        self.emit_signal(straw.FeedCategoryChangedSignal(self))
+        self.emit_signal(Event.FeedCategoryChangedSignal(self))
 
     def dump(self):
         head = {'title': self.title}
