@@ -1,8 +1,9 @@
-import straw
+import Event
+import ImageCache
 
 class FeedDataRouter(object):
     def __init__(self, feed):
-        straw.ImageCache.cache.signal_connect(straw.ImageUpdatedSignal,
+        ImageCache.cache.signal_connect(Event.ImageUpdatedSignal,
                                               self.image_updated)
         self._feed = feed
         self._stopper = None
@@ -63,7 +64,7 @@ class FeedDataRouter(object):
 
     def set_stopper(self, stopper):
         self._stopper = stopper
-        stopper.signal_connect(straw.PollingStoppedSignal,
+        stopper.signal_connect(Event.PollingStoppedSignal,
                                self._polling_stopped)
     def get_stopper(self): return self._stopper
     stopper = property(get_stopper, set_stopper)
@@ -77,7 +78,7 @@ class FeedDataRouter(object):
             self._stopper.stop()
             self._stopper = None
         for image in self._wait_images.keys():
-            straw.ImageCache.cache.stop_transfer(image)
+            ImageCache.cache.stop_transfer(image)
             del self._wait_images[image]
         self.update_process_status()
         self._feed.poll_done()
