@@ -3,15 +3,19 @@
 # GNU General Public License, version 2 or later
 #
 
-import straw
+import Event
+import Feed
+import Config
 import locale
 
-class FeedList(list, straw.SignalEmitter):
+class FeedList(list, Event.SignalEmitter):
     def __init__(self, init_seq = []):
-        straw.SignalEmitter.__init__(self)
-        self.initialize_slots(straw.FeedsChangedSignal,
-                              straw.FeedDeletedSignal,
-                              straw.FeedCreatedSignal)
+        Event.SignalEmitter.__init__(self)
+        self.initialize_slots(Event.FeedsChangedSignal,
+                              Event.FeedDeletedSignal,
+                              Event.FeedCreatedSignal,
+                              Event.AllItemsReadSignal,
+                              Event.ItemReadSignal)
         self._loading = False
 
     def load_data(self):
@@ -23,20 +27,22 @@ class FeedList(list, straw.SignalEmitter):
                     #self.append(parent, fc)
                     _load(df[1:], parent)
                 else:
-                    f = straw.create_empty_feed()
+                    f = Feed.Feed.create_empty_feed()
                     f.undump(df)
                     self.append(parent, f)
         self._loading = True
-        feeds = straw.Config.get_instance().feeds
+        feeds = Config.get_instance().feeds
         if feeds is not None:
             _load(feeds, None)
         self._loading = False
-        self.emit_signal(straw.FeedsChangedSignal(self))
+        self.emit_signal(Event.FeedsChangedSignal(self))
 
     def connect_signals(self, ob):
-        ob.signal_connect(straw.SaveFeedsSignal, self.feeds_changed, False)
-        ob.signal_connect(straw.FeedsChangedSignal, self.feeds_changed, True)
-        ob.signal_connect(straw.FeedErrorStatusChangedSignal, self.feeds_changed, False)
+        ob.signal_connect(Event.SaveFeedsSignal, self.feeds_changed, False)
+        ob.signal_connect(Event.FeedsChangedSignal, self.feeds_changed, True)
+        ob.signal_connect(Event.FeedErrorStatusChangedSignal, self.feeds_changed, False)
+        ob.signal_connect(Event.AllItemsReadSignal, lambda signal: self.emit_signal(signal))
+        ob.signal_connect(Event.ItemReadSignal, lambda signal: self.emit_signal(signal))
 
     def __setitem__(self, key, value):
         list.__setitem__(self, key, value)
@@ -48,14 +54,14 @@ class FeedList(list, straw.SignalEmitter):
         value.parent = parent
         self.connect_signals(value)
         self.save_feeds_and_notify(
-            True, signal=straw.FeedCreatedSignal(self, value, parent))
+            True, signal=Event.FeedCreatedSignal(self, value, parent))
 
     def insert(self, index, parent, value):
         list.insert(self, index, value)
         value.parent = parent
         self.connect_signals(value)
         self.save_feeds_and_notify(
-            True, signal=straw.FeedCreatedSignal(self, value, parent, index))
+            True, signal=Event.FeedCreatedSignal(self, value, parent, index))
 
     def reorder(self, move, delta):
         k = self[:]
@@ -75,18 +81,21 @@ class FeedList(list, straw.SignalEmitter):
         feed = self[value]
         list.__delitem__(self, value)
         self.save_feeds_and_notify(
-            True, signal=straw.FeedDeletedSignal(self, feed))
+            True, signal=Event.FeedDeletedSignal(self, feed))
+
+    def __del__(self):
+        self.save_feeds_and_notify(False)
 
     def save_feeds_and_notify(self, notify, signal=None):
         if not self._loading:
-            straw.Config.get_instance().feeds = [f.dump() for f in self]
+            Config.get_instance().feeds = [f.dump() for f in self]
             if notify:
                 if signal is None:
-                    signal = straw.FeedsChangedSignal(self)
+                    signal = Event.FeedsChangedSignal(self)
                 self.emit_signal(signal)
 
     def feeds_changed(self, signal, notify):
-        self.save_feeds_and_notify(notify)
+        self.save_feeds_and_notify(notify, signal)
 
     def sort(self, indices = None):
         if not indices or len(indices) == 1:
