@@ -1,5 +1,11 @@
-import straw
+import NetworkConstants
+import Event
+import URLFetch
+import PollManager
+import ItemStore
+import utils
 import error
+import Config
 
 STATUS_IMAGE_WAITING = None
 STATUS_IMAGE_BROKEN = None
@@ -25,18 +31,18 @@ class CacheEntry(object):
     def set_pollstopper(self, pollstopper):
         self._pollstopper = pollstopper
         if pollstopper is not None:
-            pollstopper.signal_connect(straw.PollingStoppedSignal,
+            pollstopper.signal_connect(Event.PollingStoppedSignal,
                                        self._polling_stopped)
     pollstopper = property(get_pollstopper, set_pollstopper)
 
     def _polling_stopped(self):
         self._pollstopper = None
 
-class Cache(straw.SignalEmitter):
+class Cache(Event.SignalEmitter):
     """Image cache with explicit reference counting."""
     def __init__(self):
-        straw.SignalEmitter.__init__(self)
-        self.initialize_slots(straw.ImageUpdatedSignal)
+        Event.SignalEmitter.__init__(self)
+        self.initialize_slots(Event.ImageUpdatedSignal)
         # self.__cache contains items of the form [Image, refcount]
         self.__cache = {}
 
@@ -53,17 +59,17 @@ class Cache(straw.SignalEmitter):
                 if item and item.feed:
                     headers['Referer'] = item.feed.location
                 try:
-                    stopper = straw.URLFetch.connection_manager.request(
+                    stopper = URLFetch.connection_manager.request(
                         key, ic,
-                        priority=straw.NetworkConstants.PRIORITY_IMAGE,
+                        priority=NetworkConstants.PRIORITY_IMAGE,
                         headers=headers)
-                except straw.URLFetch.RequestSchemeException, e:
+                except URLFetch.RequestSchemeException, e:
                     ic.http_failed(e)
                 except Exception, e:
                     error.logtb("ImageCache.add_refer:  ", str(e))
                     ic.http_failed(e)
                 self.__cache[key] = CacheEntry(
-                    image, 1, straw.PollManager.PollStopper(stopper, image))
+                    image, 1, PollManager.PollStopper(stopper, image))
             else:
                 image = Image(key, Image.DATA_IN_DB)
                 self.__cache[key] = CacheEntry(image, 1)
@@ -74,11 +80,11 @@ class Cache(straw.SignalEmitter):
         self.__cache[key].decr_count()
         if self.__cache[key].count == 0:
             del self.__cache[key]
-            self.emit_signal(straw.ImageUpdatedSignal(self, key, None))
+            self.emit_signal(Event.ImageUpdatedSignal(self, key, None))
 
     def image_updated(self, url, data):
         self.__cache[url].pollstopper = None
-        self.emit_signal(straw.ImageUpdatedSignal(self, url, data))
+        self.emit_signal(Event.ImageUpdatedSignal(self, url, data))
 
     def set_image(self, url, image):
         if self.__cache.has_key(url):
@@ -120,7 +126,8 @@ class Image:
         self.status = self.FAILED
 
     def read_data(self):
-        return straw.main._itemstore.read_image(self.url)
+        c = Config.get_instance()
+        return ItemStore.get_instance(c.straw_dir).read_image(self.url)
 
     def _return_id(self):
         return "%d %s" % (id(self), self.url)
@@ -146,7 +153,8 @@ class ImageConsumer:
 def initialize():
     global STATUS_IMAGE_WAITING
     global STATUS_IMAGE_BROKEN
+    libdir = utils.find_lib_dir()
     STATUS_IMAGE_WAITING = open(
-        straw.main._libdir + "/image-waiting.png").read()
+        libdir + "/image-waiting.png").read()
     STATUS_IMAGE_BROKEN = open(
-        straw.main._libdir + "/image-broken.png").read()
+        libdir + "/image-broken.png").read()
