@@ -2,6 +2,8 @@ import cPickle as pickle
 import os, sys
 import time
 from error import *
+import pygtk
+pygtk.require('2.0')
 import gtk
 import tempfile
 import traceback
@@ -13,7 +15,11 @@ except ImportError:
     from bsddb3.db import *
     import bsddb3 as bsddb
 
-import straw
+import FeedList
+import Event
+import SummaryItem
+import hig_alert
+import ImageCache
 
 DATABASE_FILE_NAME = "itemstore.db"
 
@@ -36,6 +42,7 @@ class MyDB:
         self._env = DBEnv()
         self._env.set_lk_detect(DB_LOCK_DEFAULT)  # enable auto deadlock avoidance
         self._env.set_lg_max(2**20)
+        #self._env.set_cachesize(0, 600000, 1)
 
         try:
             self._env.open(dbhome, flagsforenv | recoverenv, mode)
@@ -71,7 +78,7 @@ class MyDB:
             fh = open(filename, "w")
             traceback.print_exc(None, fh)
             fh.close()
-            straw.hig_alert.reportError(
+            hig_alert.reportError(
                 _("There was a problem while converting the database"),
                 _("""
                 Straw will not behave as excepted, you should probably quit now.
@@ -91,7 +98,7 @@ class MyDB:
             self._env = None
 
     def checkpoint(self):
-        # set flags arg to 0 or DB_FORCE, if not, raises EINVAL (InvalidArgError)
+        # set flags to 0 or DB_FORCE, else will raise EINVAL (InvalidArgError)
         cpflags = 0
         self._env.txn_checkpoint(cpflags | DB_FORCE )
         deletees = self._env.log_archive(DB_ARCH_ABS)
@@ -103,6 +110,16 @@ class MyDB:
 
     def get_feed_item_ids(self, feed, txn=None):
         key = "fids:%d" % feed.id
+        return self.__get_item_ids(key, txn)
+
+    def get_item_ids_for_id(self, iid, txn=None):
+        # Used for low-level stuff
+        # Hopefully, this will be used in the future for more separation
+        # Currently used by config conversion
+        key = "fids:%d" % iid
+        return self.__get_item_ids(key, txn)
+
+    def __get_item_ids(self, key, txn):
         dids = self._db.get(key, txn=txn)
         ids = []
         if dids:
@@ -129,12 +146,23 @@ class MyDB:
             raise
 
     def get_item(self, feed, item_id, txn=None):
+        return self.__get_item(feed.id, item_id, txn)
+
+    def get_item_for_id(self, iid, item_id, txn=None):
+        # Used for low-level stuff
+        # Hopefully, this will be used in the future so we can separate
+        # mechanism from policy
+        # Currently by config conversion
+        return self.__get_item(iid, item_id, txn)
+
+    def __get_item(self, iid, item_id, txn=None):
         item = unstringify_item(
-            self._db.get("%d:%d" % (feed.id, item_id), txn=txn))
+            self._db.get("%d:%d" % (iid, item_id), txn=txn))
         if item is None:
             return None
         return item
 
+
     def add_item(self, item):
         txn = self.begin_transaction()
         try:
@@ -143,10 +171,11 @@ class MyDB:
             self.save_feed_item_ids(feed, feed_ids, txn)
             self._db.put("%d:%d" % (item.feed.id, item.id),
                          stringify_item(item), txn=txn)
-            txn.commit()
-        except:
+        except Exception, ex:
             txn.abort()
-            raise
+            logtb(str(ex))
+        else:
+            txn.commit()
 
     def add_items(self, feed, items):
         txn = self.begin_transaction()
@@ -157,10 +186,11 @@ class MyDB:
                 self._db.put("%d:%d" % (item.feed.id, item.id),
                              stringify_item(item), txn=txn)
             self.save_feed_item_ids(feed, feed_item_ids, txn)
-            txn.commit()
-        except:
+        except Exception, ex:
             txn.abort()
-            raise
+            logtb(str(ex))
+        else:
+            txn.commit()
 
     def del_item(self, feed, item):
         txn = self.begin_transaction()
@@ -170,10 +200,11 @@ class MyDB:
                 feed_item_ids.remove(item.id)
                 self.save_feed_item_ids(feed, feed_item_ids, txn)
                 self._db.delete("%d:%d" % (feed.id, item.id), txn=txn)
-            txn.commit()
-        except:
+        except Exception, ex:
             txn.abort()
-            raise
+            logtb(str(ex))
+        else:
+            txn.commit()
 
     def modify_items(self, items):
         txn = self.begin_transaction()
@@ -181,10 +212,11 @@ class MyDB:
             for item in items:
                 self._db.put("%d:%d" % (item.feed.id, item.id),
                              stringify_item(item), txn=txn)
-            txn.commit()
-        except:
+        except Exception, ex:
             txn.abort()
-            raise
+            logtb(str(ex))
+        else:
+            txn.commit()
 
     def get_feed_items(self, feed):
         txn = self.begin_transaction()
@@ -195,11 +227,34 @@ class MyDB:
                 item = self.get_item(feed, id, txn=txn)
                 if item is not None:
                     items.append(item)
-            txn.commit()
-        except:
+        except Exception, ex:
             txn.abort()
-            raise
-        return items
+            logtb(str(ex))
+        else:
+            txn.commit()
+            return items
+
+    def get_number_of_unread(self, fid, cutoff):
+        # Used by config conversion
+        # NOTE: this is the number of unread items in 'number of items stored'
+        # preference. Since straw stores the most recent items down the list,
+        # we only count the unread items from the most recent N items,
+        # where N = cutoff.
+        txn = self.begin_transaction()
+        num_unread = 0
+        try:
+            ids = self.get_item_ids_for_id(fid, txn=txn)
+            for id in ids[len(ids)-cutoff:]:
+                item = self.get_item_for_id(fid, id, txn=txn)
+                if item is not None and item.seen == 0:
+                    num_unread += 1
+                else: continue
+        except Exception, ex:
+            txn.abort()
+            logtb(str(ex))
+        else:
+            txn.commit()
+            return num_unread
 
     def get_image_urls(self, txn=None):
         dkeys = self._db.get("images", txn=txn)
@@ -373,32 +428,31 @@ class ImageUpdateAction:
         db.update_image(self._url, self._image)
 
 class ItemStore:
-    def __init__(self):
-        config = straw.Config.get_instance()
-        feedlist = straw.FeedList.get_instance()
-        self._db = MyDB(DATABASE_FILE_NAME,
-                        config._straw_dir, create = 1)
+    def __init__(self, straw_dir):
+        feedlist = FeedList.get_instance()
+        self._db = MyDB(DATABASE_FILE_NAME, straw_dir, create = 1)
         self.connect_signals()
         feedlist.signal_connect(
-            straw.FeedsChangedSignal, self.feeds_changed)
-        straw.ImageCache.cache.signal_connect(straw.ImageUpdatedSignal,
+            Event.FeedsChangedSignal, self.feeds_changed)
+        ImageCache.cache.signal_connect(Event.ImageUpdatedSignal,
                                               self.image_updated)
         self._stop = 0
         self._action_queue = []
 
-    def initialize_db(self):
-        self._db.CreateTable("items", ["feed_id", "item_id", "item_data"])
+    #def initialize_db(self):
+    #    self._db.CreateTable("items", ["feed_id", "item_id", "item_data"])
 
     def feeds_changed(self, signal):
         self.connect_signals()
 
     def connect_signals(self):
-        for f in straw.FeedList.get_instance().flatten_list():
-            f.signal_connect(straw.NewItemsSignal, self.items_added)
-            f.signal_connect(straw.ItemReadSignal, self.item_modified)
-            f.signal_connect(straw.ItemStickySignal, self.item_modified)
-            f.signal_connect(straw.AllItemsReadSignal, self.all_items_read)
-            f.signal_connect(straw.ItemDeletedSignal, self.item_deleted)
+        flist = FeedList.get_instance().flatten_list()
+        for f in flist:
+            f.signal_connect(Event.NewItemsSignal, self.items_added)
+            f.signal_connect(Event.ItemReadSignal, self.item_modified)
+            f.signal_connect(Event.ItemStickySignal, self.item_modified)
+            f.signal_connect(Event.AllItemsReadSignal, self.all_items_read)
+            f.signal_connect(Event.ItemDeletedSignal, self.item_deleted)
 
     def modify_item(self, item):
         self._action_queue.append(ModifyItemAction(item))
@@ -428,6 +482,9 @@ class ItemStore:
     def read_feed_items(self, feed):
         return self._db.get_feed_items(feed)
 
+    def get_number_of_unread(self, feed_id, cutoff):
+        return self._db.get_number_of_unread(feed_id, cutoff)
+
     def start(self):
         gtk.timeout_add(5000, self.run)
 
@@ -464,6 +521,14 @@ class ItemStore:
             self._db.checkpoint()
             self._db.close()
 
+itemstore_instance = None
+
+def get_instance(straw_dir=None):
+    global itemstore_instance
+    if itemstore_instance is None:
+        itemstore_instance = ItemStore(straw_dir)
+    return itemstore_instance
+
 def stringify_item(item):
     itemdict = {
         'title': item.title,
@@ -497,7 +562,7 @@ def unstringify_item(itemstring):
     except ValueError, e:
         log("ItemStore.unstringify_item: pickle.loads raised ValueError, argument was %s" % repr(itemstring))
         return None
-    item = straw.SummaryItem()
+    item = SummaryItem.SummaryItem()
     item.title = dict['title']
     item.link = dict['link']
     item.description = dict['description']
@@ -519,8 +584,7 @@ def unstringify_item(itemstring):
     item.publication_volume = dict.get('publication_volume', None)
     item.publication_number = dict.get('publication_number', None)
     item.publication_section = dict.get('publication_section', None)
-    item.publication_starting_page = dict.get('publication_starting_page',
-                                              None)
+    item.publication_starting_page = dict.get('publication_starting_page', None)
     return item
 
 if __name__ == '__main__':
