@@ -1,5 +1,7 @@
 import StringIO
 
+import pygtk
+pygtk.require('2.0')
 import gobject
 import gtk
 import gnome
@@ -14,9 +16,16 @@ import urlparse
 import re
 import SubscribeDialog
 from error import log, logparam, logtb
+import error
 from gtk import glade
 from xml.sax import saxutils
 
+class FeedSelectionChangedSignal(straw.BaseSignal):
+    def __init__(self, sender, old, new):
+        straw.BaseSignal.__init__(self, sender)
+        self.old = old
+        self.new = new
+
 class DummyDocStream(StringIO.StringIO):
     def write_stream(self, s):
         self.write(s)
@@ -24,6 +33,7 @@ class DummyDocStream(StringIO.StringIO):
 class ItemView:
     def __init__(self, item_view_container):
         self._item = None
+        self._feed = None
         self._encoding = straw.utils.get_locale_encoding()
         self._mmgr = straw.MessageManager.get_instance()
 
@@ -31,7 +41,13 @@ class ItemView:
         self._scrolled_window = widget_tree.get_widget('html_scrolled_window')
         self._href_widget = gnome.ui.HRef(url='', text='')
         self._htmlview = gtkhtml2.View()
-        self._htmlview.set_property('can-focus', False)
+
+        # Make the article view focusable. gtkhtml2.View scrolls to the
+        # first link of the document which sometimes makes the first part of
+        # the document unviewable, esp. if the first link is found at the
+        # bottom part of the document.
+        self._htmlview.set_property('can-focus', True)
+
         self._document = gtkhtml2.Document()
         self._document.clear()
 
@@ -51,17 +67,20 @@ class ItemView:
         widget_tree.signal_connect('on_source_subscribe_button_clicked',
                                    self.on_source_subscribe_button_clicked)
 
+        self._css = None
+
+    def feed_selection_changed(self, old, new):
+        self._feed = new
+
     def on_url(self, view, url):
         if url is not None:
-            feed = straw.main.get_visible_feed()
-            url = straw.utils.complete_url(url, feed.location)
+            url = straw.utils.complete_url(url, self._feed.location)
             self._mmgr.post_message(url)
         return
 
     def link_clicked(self, document, link):
         link = link.strip()
-        feed = straw.main.get_visible_feed()
-        link = straw.utils.complete_url(link, feed.location)
+        link = straw.utils.complete_url(link, self._feed.location)
 
         try:
             gnome.url_show(link)
@@ -73,8 +92,7 @@ class ItemView:
 
     def request_url(self, document, url, stream):
         try:
-            feed = straw.main.get_visible_feed()
-            url = straw.utils.complete_url(url, feed.location)
+            url = straw.utils.complete_url(url, self._feed.location)
             if urlparse.urlparse(url)[0] == 'file':
                 # local URL, don't use the cache.
                 f = file(urlparse.urlparse(url)[2])
@@ -90,8 +108,8 @@ class ItemView:
         return
 
     def on_source_subscribe_button_clicked(self, button, *data):
-        f = straw.create_new_feed(self._item.source['text'],
-                                  location = self._item.source['url'])
+        f = straw.Feed.create_new_feed(self._item.source['text'],
+                                       location = self._item.source['url'])
         category = straw.main.get_visible_category()
         fl = straw.FeedList.get_instance()
         if (category is not None) and (category not in straw.FeedCategoryList.get_instance().pseudo_categories):
@@ -100,6 +118,12 @@ class ItemView:
             fl.append(None, f)
         straw.main.poll([f])
 
+    def get_css(self):
+        if self._css is None:
+            self._css = file(os.path.join(
+                utils.find_lib_dir(), "straw.css")).read()
+        return self._css
+
     def display_item(self, item):
         self._item = item
         self._document.clear()
@@ -108,12 +132,15 @@ class ItemView:
             """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
                <html>
                <head><title>title</title>
-            <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
-               <link rel="stylesheet" type="text/css" href="file://""" +
-               os.path.join(utils.find_lib_dir(), "straw.css") + """" />
-               </head><body>""" +
-                self.htmlify_item(item) +
-                "</body></html>")
+            <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />""")
+        config = straw.Config.get_instance()
+        if config.reload_css:
+            html += (
+                """<link rel="stylesheet" type="text/css" href="file://""" +
+                os.path.join(utils.find_lib_dir(), "straw.css") + """" />""")
+        else:
+            html += '<style type="text/css">' + self.get_css() + '</style>'
+        html += "</head><body>" + self.htmlify_item(item) + "</body></html>"
         html = html.encode('utf-8')
 	self._document.write_stream(html)
         self._document.close_stream()
@@ -146,24 +173,24 @@ class ItemView:
                 "</p></body></html>")
 
     def htmlify_item(self, item):
-        feed = straw.main.get_visible_feed()
+        feed = self._feed
         ret = []
-        ret.append('<table class="itemhead" cellspacing="0" cellpadding="0">')
+        ret.append('<table id="itemhead" cellspacing="0" cellpadding="0">')
         if item.title is not None:
-            ret.append(''.join(('<tr><td class="headleft">', _('Title:'),
-                                '</td><td class="headright">')))
+            ret.append(''.join(('<tr><td class="headleft" id="title">',
+				_('Title:'), '</td><td class="headright"><b>')))
             if (item.link is not None):
                 ret.append('<a href="%s">%s</a>' % 
                            (item.link,item.title))
             else:
                 ret.append(item.title)
-            ret.append('</td></tr>')
+            ret.append('</b></td></tr>')
             
         if item.pub_date is not None:
             timestr = straw.utils.format_date(
                 item.pub_date, utils.get_date_format(), self._encoding)
-            ret.append(''.join(('<tr><td class="headleft">', _('Date:'),
-                                '</td><td class="headright">',
+            ret.append(''.join(('<tr><td class="headleft" id="date">',
+				_('Date:'), '</td><td class="headright">',
                                 str(timestr), '</td></tr>')))
 
         feedtitle = straw.utils.convert_entities(feed.channel_title)
@@ -174,7 +201,7 @@ class ItemView:
         feedtitle = feedtitle.strip()
         feedlink = feed.channel_link.strip()
 
-        ret.append(''.join(('<tr><td class="headleft">', _('Feed:'),
+        ret.append(''.join(('<tr><td class="headleft" id="feed">', _('Feed:'),
                             '</td><td class="headright">')))
         islink = len(feedlink) > 0
         if islink:
@@ -206,10 +233,17 @@ class ItemView:
             ret.append('</div>')
 
         # freshmeat fields
+        freshmeat_data = []
         if item.fm_license != '' and item.fm_license is not None:
-            ret.append('<div class="description"><p><b>%s:</b> %s</p>' % (_("Software license"), item.fm_license))
+            freshmeat_data.append('<p><b>%s:</b> %s</p>' %
+                                  (_("Software license"), item.fm_license))
         if item.fm_changes != '' and item.fm_changes is not None:
-            ret.append('<p><b>%s:</b> %s</p>' % (_("Changes"), item.fm_changes))
+            freshmeat_data.append('<p><b>%s:</b> %s</p>' %
+                                  (_("Changes"), item.fm_changes))
+        if len(freshmeat_data) > 0:
+            ret.append('<div class="description">')
+            ret.extend(freshmeat_data)
+            ret.append('</div>')
         # empty paragraph to make sure that we get space here
         ret.append('<p></p>')
         # Additional information
@@ -233,7 +267,7 @@ class ItemView:
                 dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><a class="dclink" href="%s"><span>%s</span></a></td></tr>' % (_("License"), l, l))
 
         if dcret != []:
-            ret.append('<div class="dcinfo">%s<table class="dc">' % _("Additional information"))
+            ret.append('<div class="dcinfo">%s<table class="dc" id="footer">' % _("Additional information"))
             ret.append("".join(dcret))
             ret.append('</table>')
         ret.append('</div>')
@@ -363,18 +397,26 @@ class ItemList:
         first_time = self._feed is None
         if not (first_time or redisplay):
             self._feed.signal_disconnect(straw.RefreshFeedDisplaySignal,
-                                         self.feed_order_changed)
+                                         self._feed_order_changed)
+            self._feed.signal_disconnect(straw.AllItemsReadSignal,
+                                         self._all_items_read)
+            self._feed.signal_disconnect(straw.ItemReadSignal,
+                                         self._item_read)
         self._feed = feed
+        if not redisplay:
+            feed.signal_connect(straw.RefreshFeedDisplaySignal,
+                                self._feed_order_changed)
+            feed.signal_connect(straw.AllItemsReadSignal,
+                                self._all_items_read)
+            feed.signal_connect(straw.ItemReadSignal,
+                                self._item_read)
         count = self.render_feed(feed)
         if (not redisplay) and count and select_first:
             self._widget.get_selection().select_path((0,))
             self._widget.scroll_to_cell((0,))
-        if not redisplay:
-            feed.signal_connect(straw.RefreshFeedDisplaySignal,
-                                self.feed_order_changed)
         return
 
-    def feed_order_changed(self, event):
+    def _feed_order_changed(self, event):
         if event.sender is self._feed:
             selection = self._widget.get_selection()
             liststore, iter = selection.get_selected()
@@ -383,6 +425,13 @@ class ItemList:
             self.render_feed(event.sender)
             selection.select_path(event.sender.get_item_index(item))
 
+    def _all_items_read(self, signal):
+        for index, item in signal.changed:
+            self.mark_item(item, index)
+
+    def _item_read(self, signal):
+        self.mark_item(signal.item)
+
     def display_empty_feed(self, feed):
         self._widget.get_model().clear()
 
@@ -549,6 +598,7 @@ class FindResultList:
         model.clear()
         self._countlabel.set_text(_('%d items found') % len(items))
         i = 0
+        mainloopmanager = straw.MainloopManager.get_instance()
         for item in items:
             if self._find_interrupted:
                 self._find_interrupted = 0
@@ -564,8 +614,7 @@ class FindResultList:
                       self.COLUMN_FEED, item.feed.title,
                       self.COLUMN_ITEM, item)
             if i % 10 == 0:
-                while gtk.events_pending():
-                    gtk.mainiteration(gtk.FALSE)
+                mainloopmanager.call_pending()
 
     def item_selection_changed(self, selection):
         s = selection.get_selected()
@@ -736,9 +785,6 @@ class FeedDisplayAdapter(DisplayAdapter):
             return (1, gtk.STOCK_DIALOG_ERROR)
         return (0, None)
 
-    def is_visible(self):
-        return self._ob is straw.main.get_visible_feed()
-
     def get_feed(self):
         return self._ob
 
@@ -746,7 +792,6 @@ class FeedDisplayAdapter(DisplayAdapter):
     unread = property(get_unread)
     contents = property(lambda x: None)
     status_icon = property(get_status_icon)
-    visible = property(is_visible)
     feed = property(get_feed)
     open = property(lambda x: None)
 
@@ -754,7 +799,7 @@ def create_adapter(ob):
     if isinstance(ob, straw.Feed):
         return FeedDisplayAdapter(ob)
 
-class FeedListView:
+class FeedListView(straw.SignalEmitter):
     COLUMN_NAME = 0
     COLUMN_UNREAD = 1
     COLUMN_BOLD = 2
@@ -765,6 +810,8 @@ class FeedListView:
     COLUMN_POPUP = 7
     
     def __init__(self, widget):
+        straw.SignalEmitter.__init__(self)
+        self.initialize_slots(FeedSelectionChangedSignal)
         widget_tree = glade.get_widget_tree(widget)
         self._widget = widget
         model = gtk.TreeStore(
@@ -795,6 +842,7 @@ class FeedListView:
         self._widget.get_selection().connect(
             "changed", self.feed_selection_changed, self.COLUMN_OBJECT)
         self.setup_dnd()
+        self._feed = None
 
     def get_widget(self):
         return self._widget
@@ -849,8 +897,7 @@ class FeedListView:
             return 1
 
     def on_menu_poll_selected_activate(self, *args):
-        feed = straw.main.get_visible_feed()
-        straw.main.poll([feed])[0]
+        straw.main.poll([self._feed])[0]
 
     def on_menu_stop_poll_selected_activate(self, *args):
         model, iter = self._widget.get_selection().get_selected()
@@ -862,7 +909,7 @@ class FeedListView:
         straw.main.mark_as_read()
 
     def display_properties_feed(self, *args):
-        straw.main.show_feed_properties(straw.main.get_visible_feed())
+        straw.main.show_feed_properties(self._feed)
 
     def get_location(self):
         selection = self._widget.get_selection()
@@ -889,9 +936,16 @@ class FeedListView:
         model, iter = selection.get_selected()
         if iter is None:
             return
+        oldfeed = self._feed
         adapter = model.get_value(iter, column)
         if adapter is not None and adapter.feed is not None:
+            self._feed = adapter.feed
+        else:
+            self._feed = None
+        self.emit_signal(FeedSelectionChangedSignal(self, oldfeed, self._feed))
+        if self._feed is not None:
             straw.main.display_feed(adapter.feed)
+            
         return
 
     def display_feed_unselected(self, feed):
@@ -902,7 +956,6 @@ class FeedListView:
         self._widget.get_selection().select_path(path)
 
     def _display_feeds(self, feeds, parent):
-        #logtb("enter")
         model = self._widget.get_model()
         for f in feeds:
             adapter = create_adapter(f)
@@ -913,9 +966,9 @@ class FeedListView:
                       self.COLUMN_OBJECT, adapter,
                       self.COLUMN_ALLOW_CHILDREN, False)
             adapter.signal_connect(straw.ItemReadSignal,
-                                straw.main_window.item_read, iter)
+                                   self._item_read, iter)
             adapter.signal_connect(straw.AllItemsReadSignal,
-                                straw.main_window.all_items_read, iter)
+                                   self._all_items_read, iter)
             adapter.signal_connect(straw.ItemsAddedSignal,
                                    straw.main_window.feed_updated, iter)
             adapter.signal_connect(straw.FeedPolledSignal,
@@ -932,7 +985,7 @@ class FeedListView:
                       self.COLUMN_STATUS_PIXBUF, pixbuf,
                       self.COLUMN_POPUP, self._feed_popup)
 
-            if adapter.visible:
+            if adapter.feed is self._feed:
                 self._widget.get_selection().select_iter(iter)
 
     def display_feeds(self, feeds, visible_feed):
@@ -945,7 +998,7 @@ class FeedListView:
         self._widget.columns_autosize()
         self.unblock_row_signals()
 
-    def all_items_read(self, signal, feed_index):
+    def _all_items_read(self, signal, feed_index):
         model = self._widget.get_model()
         new = signal.sender.unread
         weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
@@ -953,7 +1006,7 @@ class FeedListView:
         model[feed_index][self.COLUMN_BOLD] = weight
         self._widget.queue_draw()
 
-    def item_read(self, signal, iter):
+    def _item_read(self, signal, iter):
         model = self._widget.get_model()
         feed_index = model.get_path(iter)
         new = signal.sender.unread
@@ -962,19 +1015,30 @@ class FeedListView:
         model[feed_index][self.COLUMN_BOLD] = weight
         self._widget.queue_draw()
 
-    def feed_updated(self, signal, feed_index):
+    def feed_adapter_updated(self, adapter, feed_index):
         model = self._widget.get_model()
-        new = signal.sender.unread
-        #logtb("new for %s (%s): %s" % (str(signal.sender.title), model.get_path(feed_index), new))
+        new = adapter.unread
+        row = model[feed_index]
         weight = (pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD)[new[1] > 0]
-        model[feed_index][self.COLUMN_UNREAD] = new[0]
-        model[feed_index][self.COLUMN_BOLD] = weight
-        status, pixbuf = signal.sender.status_icon
-        model[feed_index][self.COLUMN_STATUS_FLAG] = status
+        row[self.COLUMN_NAME] = adapter.title
+        row[self.COLUMN_UNREAD] = new[0]
+        row[self.COLUMN_BOLD] = weight
+        status, pixbuf = adapter.status_icon
+        row[self.COLUMN_STATUS_FLAG] = status
         if pixbuf:
-            model[feed_index][self.COLUMN_STATUS_PIXBUF] = pixbuf
+            row[self.COLUMN_STATUS_PIXBUF] = pixbuf
         self._widget.queue_draw()
 
+    def feed_updated(self, feed):
+        model = self._widget.get_model()
+        # TODO: in python 2.3, use enumerate
+        for index in range(len(model)):
+            row = model[index]
+            adapter = row[self.COLUMN_OBJECT]
+            if adapter.feed is feed:
+                self.feed_adapter_updated(adapter, index)
+                break
+
     def disable_feed_selection(self):
         self._widget.set_sensitive(0)
 
@@ -1185,6 +1249,27 @@ class ErrorIndicator:
             self._tooltips.enable()
             self._widget.show()
 
+class TotalUnreadIndicator:
+    def __init__(self, window):
+        self._window = window
+        feedlist = straw.FeedList.get_instance()
+        feedlist.signal_connect(straw.AllItemsReadSignal,
+                                lambda signal: self.update())
+        feedlist.signal_connect(straw.ItemReadSignal,
+                                lambda signal: self.update())
+        self._feedlist = feedlist
+
+    def update(self):
+        uritems = urfeeds = 0
+        for ur in [f.number_of_unread for f in self._feedlist.flatten_list()]:
+            if ur:
+                uritems += ur
+                urfeeds += 1
+        map = {'uritems': uritems, 'urfeeds': urfeeds}
+        self._window.set_title(('%s - ' % straw.APPNAME) +
+                               _('%(uritems)d unread in %(urfeeds)d feeds') %
+                               map )
+
 class MainWindow:
     def __init__(self, xml):
 
@@ -1206,8 +1291,12 @@ class MainWindow:
             xml.get_widget('toolbar_default'))
         self._error_indicator = ErrorIndicator(
             xml.get_widget('statusbar_error_indicator'))
-
         self._window = xml.get_widget('straw_main')
+        self._unread_indicator = TotalUnreadIndicator(self._window)
+
+        self._feed_list_view.signal_connect(FeedSelectionChangedSignal,
+                                            self._feed_selection_changed)
+
         self._window.set_default_size(*self._config.main_window_size)
         xml.get_widget('main_main_pane').set_position(self._config.main_pane_position)
         xml.get_widget('main_sub_pane').set_position(self._config.sub_pane_position)
@@ -1239,11 +1328,16 @@ class MainWindow:
         self._feed_refresh = xml.get_widget('feed_refresh_label')
 
         self._feedlist = straw.FeedList.get_instance()
+        self._feed = None
+
+    def _feed_selection_changed(self, signal):
+        self._item_view.feed_selection_changed(signal.old, signal.new)
+        self._feed = signal.new
 
     def category_select(self, category, *args):
         straw.main.set_visible_category(category)
         self.display_feeds(
-            category, straw.main.get_visible_feed())
+            category, self._feed)
         self._error_indicator.switch_category(category)
 
     def get_find_results_view(self):
@@ -1314,15 +1408,14 @@ class MainWindow:
         straw.main.show_preferences_dialog()
 
     def on_menu_feed_properties_activate(self, *args):
-        straw.main.show_feed_properties(straw.main.get_visible_feed())
+        straw.main.show_feed_properties(self._feed)
 
     def on_menu_refresh_all_activate(self, *args):
         straw.main.poll_categories(
             straw.FeedCategoryList.get_instance().all_categories)
 
     def on_menu_refresh_selected_activate(self, *args):
-        feed = straw.main.get_visible_feed()
-        straw.main.poll([feed])
+        straw.main.poll([self._feed])
 
     def on_menu_refresh_category_activate(self, *args):
         category = straw.main.get_visible_category()
@@ -1424,7 +1517,9 @@ class MainWindow:
         self._feed_label.set_label("%s _Articles:" % feed.title)
         next = feed.next_refresh
         if next is not None:
-            self._feed_refresh.set_label(_("Next Refresh: %s") % time.strftime(utils.get_date_format(), time.localtime(next)))
+            self._feed_refresh.set_label(
+                _("Next Refresh: %s") %
+                straw.utils.format_date(time.gmtime(next)))
             self._feed_refresh.show()
         else:
             self._feed_refresh.hide()
@@ -1461,38 +1556,23 @@ class MainWindow:
         m = self._mmgr.read_message()
         self._status_view.push(cid, m)
 
-    def update_total_unread_indicator(self):
-        uritems = urfeeds = 0
-        for ur in [f.number_of_unread for f in self._feedlist.flatten_list()]:
-            if ur:
-                uritems += ur
-                urfeeds += 1
-        map = {'uritems': uritems, 'urfeeds': urfeeds}
-        self._window.set_title(('%s - ' % straw.APPNAME) +
-                               _('%(uritems)d unread in %(urfeeds)d feeds') %
-                               map )
-
-    def all_items_read(self, signal, feed_index):
-        self._feed_list_view.all_items_read(signal, feed_index)
-        for index, item in signal.changed:
-            self._item_list_view.mark_item(item, index)
-        self.update_total_unread_indicator()
-
-    def item_read(self, signal, feed_index):
-        self._feed_list_view.item_read(signal, feed_index)
-        self._item_list_view.mark_item(signal.item)
-        self.update_total_unread_indicator()
-
     def feed_updated(self, signal, feed_index):
-        self._feed_list_view.feed_updated(signal, feed_index)
-        if signal.sender.feed is straw.main.get_visible_feed():
+        self._feed_list_view.feed_adapter_updated(signal.sender, feed_index)
+        if signal.sender.feed is self._feed:
             self.display_feed(signal.sender.feed)
-        self.update_total_unread_indicator()
+        self._unread_indicator.update()
 
     def display_feeds(self, category, visible_feed):
         self._category_selector.set_selected(category)
         self._feed_list_view.display_feeds(category.feeds, visible_feed)
-        self.update_total_unread_indicator()
+        self._unread_indicator.update()
+
+    # TODO: should be merged with feed_updated, maybe display_feed? this whole thing needs a serious refactor
+    def update_feed(self, feed):
+        self._feed_list_view.feed_updated(feed)
+        if feed is self._feed:
+            self.display_feed(feed)
+        self._unread_indicator.update()
 
     def display_categories(self):
         self._category_selector.display_categories()
