@@ -1,14 +1,14 @@
-import straw
+import Event
 
-class MessageManager(straw.SignalEmitter):
+class MessageManager(Event.SignalEmitter):
     def __init__(self):
-        straw.SignalEmitter.__init__(self)
-        self.initialize_slots(straw.StatusDisplaySignal)
+        Event.SignalEmitter.__init__(self)
+        self.initialize_slots(Event.StatusDisplaySignal)
         self.__messages = []
 
     def post_message(self, message):
         self.__messages.append(message)
-        self.emit_signal(straw.StatusDisplaySignal(self))
+        self.emit_signal(Event.StatusDisplaySignal(self))
 
     def read_message(self):
         return self.__messages.pop(0)
