@@ -1,4 +1,6 @@
 import straw
+import pygtk
+pygtk.require('2.0')
 import gnome.vfs
 
 def export(title, list, fname):
