@@ -1,17 +1,7 @@
-cvs_id = "$Id: OPMLImport.py,v 1.2 2004/04/16 20:26:07 jpakaste Exp $"
-
 import OPML
 
 class BlogListEntry(object):
-    __slots__ = ('text', 'type', 'url', 'xmlurl', 'description', 'language',
-                 'title', 'version')
-
-    def __init__(self):
-        self.xmlurl = None
-        self.description = None
-        self.language = None
-        self.title = None
-        self.version = None
+    __slots__ = ('text', 'url')
 
 def _find_entries(outline):
     entries = []
@@ -24,22 +14,31 @@ def _find_entries(outline):
         url = outline.get('url', '')
         if url != '':
             e = BlogListEntry()
-            e.type = type
             e.text = text
             e.url = url
-    elif type == 'rss':
-        url = outline.get('htmlUrl', '')
+    else:
         xmlurl = outline.get('xmlUrl', '')
-        if url != '' and xmlurl != '':
-            e = BlogListEntry()
-            e.type = type
-            e.text = text
-            e.url = url
-            e.xmlurl = xmlurl
-            e.description = outline.get('description', '')
-            e.language = outline.get('language', '')
-            e.title = outline.get('title', '')
-            e.version = outline.get('version', '')
+        e = BlogListEntry()
+        e.text = text
+        if text == '':
+            title = outline.get('title', '')
+            if title == '':
+                e = None
+            e.text = title
+        if e != None:
+            if xmlurl != '':
+                # there's something in xmlurl. There's a good chance that's
+                # our feed's URL
+                e.url = xmlurl
+            else:
+                htmlurl = outline.get('htmlUrl', '')
+                if htmlurl != '':
+                    # there's something in htmlurl, and xmlurl is empty. This
+                    # might be our feed's URL.
+                    e.url = htmlurl
+                else:
+                    # nothing else to try.
+                    e = None
     if e is not None:
         entries[0:0] = [e]
     return entries
@@ -58,8 +57,9 @@ def read(stream):
     entries = find_entries(o.outlines)
     ret = []
     edict = {}
+    # avoid duplicates.
     for e in entries:
-        ek = (e.text, e.type, e.url)
+        ek = (e.text, e.url)
         edict[ek] = edict.get(ek, 0) + 1
         if edict[ek] < 2:
             ret.append(e)
