@@ -2,10 +2,8 @@
 # You may use and distribute this software under the terms of the
 # GNU General Public License, version 2 or later
 
-cvs_id = "$Id: ParsedSummary.py,v 1.2 2004/04/16 20:26:07 jpakaste Exp $"
-
 class ParsedSummary(object):
-    __slots__ = ('title', 'link', 'description', 'last_build_date',
+    __slots__ = ('title', 'link', 'description', 'last_build_date', 'contributors',
                  'managing_editor', 'web_master', 'language', 'items', 'copyright', 'creator')
 
     def __init__(self):
@@ -17,6 +15,7 @@ class ParsedSummary(object):
         self.link = ""
         self.language = ""
         self.creator = ""
+        self.contributors = list()
 
     def addItem(self, item):
         self.items.append(item)
