@@ -1,3 +1,5 @@
+import pygtk
+pygtk.require('2.0')
 import gobject
 import gtk
 import pango
@@ -31,6 +33,8 @@ class CategoryListView(MVP.WidgetView):
                                    self._category_removed)
         self._model.signal_connect(straw.FeedCategoryChangedSignal,
                                    self._category_changed)
+        self._model.signal_connect(straw.FeedCategoryListLoadedSignal,
+                                   self._categories_loaded)
         self._populate_treemodel()
 
     def _create_columns(self):
@@ -97,6 +101,9 @@ class CategoryListView(MVP.WidgetView):
     def _category_changed(self, signal):
         self._update_category_name(signal.sender)
 
+    def _categories_loaded(self, signal):
+        self._populate_treemodel()
+
     def _selection_changed(self, selection, *args):
         if self._presenter is None:
             error.log("presenter not set!")
@@ -510,9 +517,6 @@ class PreferencesDialog:
     def on_item_order_oldest_toggled(self, radio):
         pass
 
-    def create_feed_model(self):
-        pass
-
 class CategorySourceMonitors(WeakKeyDictionary):
     def disconnect(self, category):
         if self.has_key(category):
