@@ -1,8 +1,6 @@
 # dunno if this really makes any sense performance-wise, but it's a nice
 # abstraction
 
-cvs_id = "$Id: QueueDict.py,v 1.2 2004/04/16 20:26:07 jpakaste Exp $"
-
 class QDIterator:
     def __init__(self, qd):
         self._qd = qd
@@ -99,6 +97,8 @@ class OrderedKeyDict(QueueDict):
         return id
 
     def get_next_smaller_key(self, key):
+        # TODO: breaks if the key isn't there because the item has already
+        # disappeared, fix
         i = self._queue.index(key)
         id = None
         if i > 0:
