@@ -1,9 +1,11 @@
 import urlparse
+import pygtk
+pygtk.require('2.0')
 import gobject
 import gtk
 import gtk.glade
 import gettext
-import Feed
+from Feed import Feed
 import FeedList
 import feedfinder
 import error
