@@ -2,8 +2,6 @@
 # You may use and distribute this software under the terms of the
 # GNU General Public License, version 2 or later
 
-cvs_id = "$Id: SummaryItem.py,v 1.2 2004/04/16 20:26:07 jpakaste Exp $"
-
 import straw
 import time
 
@@ -14,10 +12,10 @@ class SummaryItem(object, straw.SignalEmitter):
                  '_searchable_fields', '_sticky', 'publication_name',
                  'publication_volume', 'publication_number',
                  'publication_section', 'publication_starting_page',
-                 'title_converted', 'guidislink')
+                 'title_converted', 'guidislink', 'contributors')
 
     _searchable_fields = ('title', 'description', 'fm_license', 'fm_changes',
-                          'creator')
+                          'creator', 'contributors')
 
     def __init__(self):
         straw.SignalEmitter.__init__(self)
@@ -36,7 +34,8 @@ class SummaryItem(object, straw.SignalEmitter):
         self.fm_license = None
         self.fm_changes = None
         self.creator = None
-        self.license_urls = []
+        self.contributors = list()
+        self.license_urls = list()
         self.publication_name = None
         self.publication_volume = None
         self.publication_number = None
