@@ -10,7 +10,6 @@ except:
 import error
 import straw
 import time
-import weakref
 
 def is_ip(host):
     parts = host.split(".")
@@ -27,6 +26,7 @@ def is_ip(host):
 
 class RequestSchemeException(Exception):
     def __init__(self, scheme):
+        Exception.__init__(self)
         self.scheme = scheme
         self.args = {}
 
@@ -81,19 +81,20 @@ class ConnectionManager:
                       user = user, password = password, priority = priority,
                       consumer = consumer, uri = uri)
         pc = straw.Config.get_instance().proxy_config
-        proxy = None
         if pc.use:
-            proxy = pc.ip
+            # we don't set ConsumerAdapter's proxy here because we might not
+            # yet know its ip. However, we can set the Proxy-Authorization
+            # line
             if pc.use_authentication:
                 req.headers['Proxy-Authorization'] = ('Basic %s' % base64.encodestring('%s:%s' % (pc.user, pc.password)).strip())
-        adapter = ConsumerAdapter(req, proxy=(proxy, pc.port))
+        adapter = ConsumerAdapter(req)
 
         # no lookup necessary if host is an ip address.
         # if we are using a proxy, let it handle the lookup.
         if is_ip(host):
             adapter.request.ip = host
             self._queue_request(adapter)
-        elif straw.Config.get_instance().proxy_config.use:
+        elif pc.use:
             self._queue_request(adapter)
         else:
             try:
@@ -123,16 +124,18 @@ class ConnectionManager:
             adapter.http_failed(_("Host name lookup failed"))
 
     def poll(self, timeout=0.1):
-        config = straw.Config.get_instance()
+        proxy_config = straw.Config.get_instance().proxy_config
         lookup_manager = straw.LookupManager.get_instance()
         # activate up to MAX_CONNECTIONS channels
-        while self._queue and len(asyncore.socket_map) < straw.NetworkConstants.MAX_CONNECTIONS and not config.proxy_config.is_waiting:
+        while self._queue and len(asyncore.socket_map) < straw.NetworkConstants.MAX_CONNECTIONS and not proxy_config.is_waiting:
             adapter = self._queue.pop(0)
             # has the user switched off the proxy after this request was queued
-            if (not config.proxy_config.use) and (not adapter.request.ip):
+            if (not proxy_config.use) and (not adapter.request.ip):
                 lookup_manager.lookup(
                     adapter.request.host, self._request_resolved, adapter)
             else:
+                if proxy_config.use:
+                    adapter.set_proxy((proxy_config.ip, proxy_config.port))
                 adapter.start()
         # keep the network running
         now = time.time()
@@ -158,9 +161,10 @@ class ConsumerAdapter(object):
     CREATED = 0
     STARTED = 1
     CLOSED = 2
-    
-    def __init__(self, req, proxy=None):
-        self.connection = httplib_async.HTTPConnection_async(req.host, req.ip, req.port, self, proxy=proxy)
+
+    # TODO: we do nothing with the proxy?
+    def __init__(self, req):
+        self.connection = httplib_async.HTTPConnection_async(req.host, req.ip, req.port, self)
         self.consumer = req.consumer
         self._data = ""
         self.header = None
@@ -189,6 +193,9 @@ class ConsumerAdapter(object):
             self._close_connection()
             self.state = self.CLOSED
 
+    def set_proxy(self, proxy):
+        self.connection.set_proxy(proxy)
+
     def time_exceeded(self):
         if self.state != self.STARTED:
             return
