@@ -1,3 +1,5 @@
+import pygtk
+pygtk.require('2.0')
 import gtk
 import gnome
 import straw
