@@ -21,10 +21,16 @@ class HTTPConnection_async(asyncore.dispatcher_with_send):
         self.consumer = consumer
         self.status = None
         self.http_version = "HTTP/1.1"
+        self.proxy = None
+        self.proxy_port = None
         if proxy:
             self.proxy = proxy[0]
             self.proxy_port = proxy[1]
 
+    def set_proxy(self, proxy):
+        self.proxy = proxy[0]
+        self.proxy_port = proxy[1]
+
     def request(self, method, path, headers={}, data=None):
         self.header = None
         self.data = ""
