@@ -116,7 +116,11 @@ def get_locale_encoding():
         encoding = sys.getdefaultencoding()
     return encoding
 
-def format_date(date, format, encoding):
+def format_date(date, format=None, encoding=None):
+    if format is None:
+        format = get_date_format()
+    if encoding is None:
+        encoding = get_locale_encoding()
     timestr = time.strftime(format, time.localtime(calendar.timegm(date)))
     return unicode(timestr, encoding)
 
@@ -124,6 +128,15 @@ def get_date_format():
     # this is here just to make xgettext happy: it should be defined in
     # only one place, and a good one would be MainWindow.py module level.
     # however, we can't access _ there.
+    # The format: %A is the full weekday name
+    #             %B is the full month name
+    #             %e is the day of the month as a decimal number,
+    #                without leading zero
+    # This should be translated to be suitable for the locale in
+    # question, feel free to alter the order and the parameters (they
+    # are strftime(3) parameters, the whole string is passed to the
+    # function, Straw does no additional interpretation) if you feel
+    # it's necessary in the translation file.
     return _('%A %B %e %H:%M')
 
 MAX_DISPLAY_CHAR = 60
